import React, { useContext } from 'react';
import { defaultProps } from '../../../VelocityAnimation/VelocityAnimation/props/defaultProps';
import { propTypes } from '../../../VelocityAnimation/VelocityAnimation/props/propTypes';
import { VelocityComponent } from 'velocity-react';
import 'velocity-animate/velocity.ui';
import LibraryContext from '../../../Provider/LibraryContextInit';
export default function VelocityAnimation(props) {
  let {
    children,
    name,
    duration,
    isActive,
    runOnMount,
    delay,
    isCustomized,
    needUIPack,
    isFlex
  } = props;
  let {
    direction,
    isReducedMotion
  } = useContext(LibraryContext) || {};
  let enterAnimationObj = {
    fade: 'transition.fadeIn',
    slideDown: 'slideDown',
    flyDown: {
      translateY: '0%'
    },
    slideRight: {
      translateX: '0%'
    },
    slideLeft: {
      translateX: '0%'
    },
    shrink: 'transition.shrinkIn',
    expand: 'transition.expandIn'
  };
  let exitAnimationObj = {
    fade: 'transition.fadeOut',
    slideDown: 'slideUp',
    flyDown: {
      translateY: '-100%'
    },
    slideRight: {
      translateX: direction == 'rtl' ? '-100%' : '100%'
    },
    slideLeft: {
      translateX: direction == 'rtl' ? '100%' : '-100%'
    },
    shrink: 'transition.shrinkOut',
    expand: 'transition.expandOut'
  };
  let animation = isCustomized ? isActive ? enterAnimationObj[name] : exitAnimationObj[name] : needUIPack ? `transition.${name}${isActive ? 'In' : 'Out'}` : name;
  return /*#__PURE__*/React.createElement(VelocityComponent, {
    key: name,
    animation: animation,
    duration: !isReducedMotion ? duration : 0,
    runOnMount: runOnMount,
    display: isActive ? isFlex ? 'flex' : '' : 'none',
    delay: delay,
    interruptBehavior: "queue"
  }, children);
}
VelocityAnimation.defaultProps = defaultProps;
VelocityAnimation.propTypes = propTypes; // if (__DOCS__) {
//   VelocityAnimation.docs = {
//     componentGroup: 'Animation'
//   };
// }